@@ -1,12 +1,12 @@
 ## better for development to avoid name conflicts with OKsmooth :toCanonical and :canonize
-canonizeRanPars <- function(ranPars, ## should have a RHOMAX attribute when trRho in input
+.canonizeRanPars <- function(ranPars, ## should have a RHOMAX attribute when trRho in input
                             corr.model,checkComplete=TRUE) {
   trueCorrpars <- list()
   if (is.null(corr.model)) {
     ## do nothing
   } else if (corr.model %in% c("Matern")) {
     if (!is.null(ranPars$trNu)) { ## either we have nu,rho or trNu,trRho 
-      ranPars$nu <- nuInv(ranPars$trNu,ranPars$trRho,NUMAX=attr(ranPars,"NUMAX")) ## before trRho is removed...
+      ranPars$nu <- .nuInv(ranPars$trNu,ranPars$trRho,NUMAX=attr(ranPars,"NUMAX")) ## before trRho is removed...
       ranPars$trNu <- NULL
       attr(ranPars,"type")$nu <- attr(ranPars,"type")$trNu
       attr(ranPars,"type")$trNu <- NULL
@@ -20,7 +20,7 @@ canonizeRanPars <- function(ranPars, ## should have a RHOMAX attribute when trRh
   } 
   if (is.null(corr.model)) {
     ## do nothing
-  } else if (corr.model %in% c("AR1","ar1")) {
+  } else if (corr.model %in% c("AR1")) {
     ARphi <- ranPars$ARphi
     if (is.null(ARphi) && checkComplete) {
       mess <- pastefrom("ARphi missing from ranPars.",prefix="(!) From ")
@@ -29,7 +29,7 @@ canonizeRanPars <- function(ranPars, ## should have a RHOMAX attribute when trRh
     trueCorrpars$ARphi <- ARphi    
   } else if (corr.model != "corrMatrix") { ## all models with a 'rho' parameter
     if (!is.null(ranPars$trRho)) { ## assuming a single trRho with possibly several elements
-      ranPars$rho <- rhoInv(ranPars$trRho,RHOMAX=attr(ranPars,"RHOMAX"))  
+      ranPars$rho <- .rhoInv(ranPars$trRho,RHOMAX=attr(ranPars,"RHOMAX"))  
       ranPars$trRho <- NULL
       attr(ranPars,"type")$rho <- attr(ranPars,"type")$trRho
       attr(ranPars,"type")$trRho <- NULL
@@ -48,21 +48,18 @@ canonizeRanPars <- function(ranPars, ## should have a RHOMAX attribute when trRh
   Nugget <- ranPars$Nugget
   if (! is.null(Nugget)) trueCorrpars$Nugget <- Nugget 
   if (!is.null(ranPars$trPhi)) {
-    ranPars$phi <- dispInv(ranPars$trPhi)
+    ranPars$phi <- .dispInv(ranPars$trPhi)
     ranPars$trPhi <- NULL
     attr(ranPars,"type")$phi <- attr(ranPars,"type")$trPhi
     attr(ranPars,"type")$trPhi <- NULL
     if (spaMM.getOption("wDEVEL2")) {
       parlist <- attr(ranPars,"parlist")
-      parlist$phi <- dispInv(parlist$trPhi)
+      parlist$phi <- .dispInv(parlist$trPhi)
       parlist$trPhi <- NULL
       attr(parlist,"types")$phi <- attr(parlist,"types")$trPhi
       attr(parlist,"types")$trPhi <- NULL
       attr(ranPars,"parlist") <- parlist
     }
-  } else if (!is.null(ranPars$logphi)) { ## debug code
-    ## HL.info$ranFix$phi <- exp(ranPars$logphi)
-    stop("logphi in HLCor...")
   } # else ranPars$phi unchanged
   # ranPars may have trLambda and lambda (eg fitme(...lambda=c(<value>,NA)))  
   # It may have $trLambda (from notlambda) for what is optimized,
@@ -72,15 +69,15 @@ canonizeRanPars <- function(ranPars, ## should have a RHOMAX attribute when trRh
     ## cf HLCor_body code at this point Fix and init.HLfit are merged 
     ##    and the type info will be used by HLCor_body to separate them
     if (is.null(lambda)) { ## only trLambda, not lambda
-      ranPars$lambda <- dispInv(ranPars$trLambda)
+      ranPars$lambda <- .dispInv(ranPars$trLambda)
       type <- attr(ranPars,"type")$trLambda 
     } else { ## merge lambda and trLambda
       len_lam <- seq(length(lambda))
       type <- attr(ranPars,"type")$lambda # presumably "fix", or "var"<=> from init.HLfit 
-      if (is.null(names(lambda))) names(type) <- names(lambda) <- len_lam ## FIXME a guess
-      fromTr <- dispInv(ranPars$trLambda)
+      if (is.null(names(lambda))) names(lambda) <- len_lam ## but do not try to assign a vector of names for a single 'type' value
+      fromTr <- .dispInv(ranPars$trLambda)
       lambda[names(fromTr)] <- fromTr  
-      type[names(fromTr)] <- attr(ranPars,"type")$trLambda ## presumably "fix" (for fully fix or outer estimated)
+      #type[names(fromTr)] <- attr(ranPars,"type")$trLambda ## presumably "fix" (for fully fix or outer estimated)
       ranPars$lambda <- lambda
     }
     attr(ranPars,"type")$lambda <- type
@@ -91,12 +88,12 @@ canonizeRanPars <- function(ranPars, ## should have a RHOMAX attribute when trRh
       lambda <- parlist$lambda
       if (is.null(lambda)) { ## only trLambda, not lambda
         types <- attr(parlist,"types")$trLambda 
-        parlist$lambda <- dispInv(parlist$trLambda)
+        parlist$lambda <- .dispInv(parlist$trLambda)
       } else { ## merge lambda and trLambda
         len_lam <- seq(length(lambda))
         types <- attr(parlist,"types")$lambda # presumably "fix", or "var"<=> from init.HLfit 
-        if (is.null(names(lambda))) names(types) <- names(lambdas) <- len_lam ## FIXME a guess
-        fromTr <- dispInv(parlist$trLambda)
+        if (is.null(names(lambda))) names(lambdas) <- len_lam 
+        fromTr <- .dispInv(parlist$trLambda)
         lambda[names(fromTr)] <- fromTr  
         types[names(fromTr)] <- attr(parlist,"types")$trLambda ## presumably "fix" (for fully fix or outer estimated)
         parlist$lambda <- lambda
