@@ -40,7 +40,8 @@
     if ( ! is.null(mc$resid.formula)) mc$resid.model <- mc$resid.formula
     names_nondefault  <- intersect(names(mc),names_FHF) ## mc including dotlist
     FHF[names_nondefault] <- mc[names_nondefault] ##  full HLfit args
-    preprocess.formal.args <- FHF[which(names_FHF %in% names(formals(preprocess)))] 
+    preprocess.formal.args <- FHF[which(names_FHF %in% names(formals(.preprocess)))] 
+    preprocess.formal.args$For <- "fitme"
     preprocess.formal.args$family <- family ## already checked 
     preprocess.formal.args$rand.families <- FHF$rand.family ## because preprocess expects $rand.families 
     preprocess.formal.args$predictor <- FHF$formula ## because preprocess stll expects $predictor 
@@ -58,15 +59,14 @@
         familyargs <- family
         familyargs$family <- NULL
         familyargs$binfamily <- NULL
-        ## we need the data list in the corrHLfit envir for the call to makeCheckGeoMatrices
+        ## we need the data list in the corrHLfit envir for the call to .makeCheckGeoMatrices
         preprocess.formal.args$data <- do.call(binomialize,c(list(data=data),familyargs)) ## if data not already binomialized
       }     
     }
-    mc$processed <- do.call(preprocess,preprocess.formal.args,envir=parent.frame(1L))
-    mc$verbose <- .reformat_verbose(eval(mc$verbose),For="corrHLfit")
+    mc$processed <- do.call(.preprocess,preprocess.formal.args,envir=parent.frame(1L))
     ## removing all elements that are matched in processed:
     pnames <- c("data","family","formula","prior.weights","HLmethod","rand.family","control.glm","resid.formula","REMLformula",
-                "resid.model")
+                "resid.model", "verbose")
     for (st in pnames) mc[st] <- NULL 
   }  
   
@@ -105,7 +105,7 @@
   attr(hlcor,"HLCorcall") <- NULL
   #class(hlcor) <- c(class(hlcor,"fitme"))
   lsv <- c("lsv",ls())
-  hlcor$fit_time <- .timerraw(time1)
+  if ( ! identical(paste(family[[1L]]),"multi"))  hlcor$fit_time <- .timerraw(time1)
   rm(list=setdiff(lsv,"hlcor")) ## empties the whole local envir except the return value
   return(hlcor)
 }
@@ -161,7 +161,8 @@ fitme <- function(formula,data, ## matches minimal call of HLfit
     if ( ! is.null(mc$resid.formula)) mc$resid.model <- mc$resid.formula
     names_nondefault  <- intersect(names(mc),names_FHF) ## mc including dotlist
     FHF[names_nondefault] <- mc[names_nondefault] ##  full HLfit args
-    preprocess.formal.args <- FHF[which(names_FHF %in% names(formals(preprocess)))] 
+    preprocess.formal.args <- FHF[which(names_FHF %in% names(formals(.preprocess)))] 
+    preprocess.formal.args$For <- "fitme"
     preprocess.formal.args$family <- family ## already checked 
     preprocess.formal.args$rand.families <- FHF$rand.family ## because preprocess expects $rand.families 
     preprocess.formal.args$predictor <- FHF$formula ## because preprocess stll expects $predictor 
@@ -179,15 +180,14 @@ fitme <- function(formula,data, ## matches minimal call of HLfit
         familyargs <- family
         familyargs$family <- NULL
         familyargs$binfamily <- NULL
-        ## we need the data list in the corrHLfit envir for the call to makeCheckGeoMatrices
+        ## we need the data list in the corrHLfit envir for the call to .makeCheckGeoMatrices
         preprocess.formal.args$data <- do.call(binomialize,c(list(data=data),familyargs)) ## if data not already binomialized
       }     
     }
-    mc$processed <- do.call(preprocess,preprocess.formal.args,envir=parent.frame(1L))
-    mc$verbose <- .reformat_verbose(eval(mc$verbose),For="corrHLfit")
+    mc$processed <- do.call(.preprocess,preprocess.formal.args,envir=parent.frame(1L))
     ## removing all elements that are matched in processed:
     pnames <- c("data","family","formula","prior.weights","HLmethod","rand.family","control.glm","resid.formula","REMLformula",
-                "resid.model")
+                "resid.model", "verbose")
     for (st in pnames) mc[st] <- NULL 
   }  
   
@@ -198,7 +198,8 @@ fitme <- function(formula,data, ## matches minimal call of HLfit
   attr(hlcor,"HLCorcall") <- NULL
   #class(hlcor) <- c(class(hlcor,"fitme"))
   lsv <- c("lsv",ls())
-  hlcor$fit_time <- .timerraw(time1)
+  if ( ! identical(paste(family[[1L]]),"multi")
+       && ! identical(.getProcessed(processed,"verbose",from=1L)["getCall"],TRUE))  hlcor$fit_time <- .timerraw(time1)
   rm(list=setdiff(lsv,"hlcor")) ## empties the whole local envir except the return value
   return(hlcor)
 }
