@@ -1,10 +1,9 @@
-slashTerms <-
-function (x) 
+.slashTerms <- function (x) 
 {
     if (!("/" %in% all.names(x))) 
         return(x)
     if (x[[1]] != as.name("/")) 
         stop("unparseable formula for grouping factor")
-    list(slashTerms(x[[2]]), slashTerms(x[[3]]))
+    list(.slashTerms(x[[2]]), .slashTerms(x[[3]]))
 }
 
