@@ -2,7 +2,7 @@
   envir <- attr(mat,"envir") ## environment
   envir$callcount <- envir$callcount+1L
   if (is.null(envir$qr)) { 
-    if (provide) envir$qr <- QRwrap(mat)
+    if (provide) envir$qr <- .QRwrap_but_diag(mat)
   } 
   return(envir$qr)
 }
